#include <system.h>
#include "ZigBeelib.h"
#include "LCDlib_SPI.h"
#include "RTClib.h"
#include "ADlib.h"
#include "usartlib.h"
#include "EEPROMlib.h"
#include "motorlib.h"

//////////////////////////////////////////
//	Smart Windows Project				//
//										//
//	ADlib.c								//
//  Functions for A/D conversion		//
//////////////////////////////////////////


// Function Prototypes in ADlib.h


////////////////////////////////////////////////
//	Function: ad_init                          
//	Purpose: Configures the A/D converter module 
//	Input: the channel you want to use             
//	Output: Void                       		       
////////////////////////////////////////////////
void ad_init(char channel){
	char temp;	// Temporary value
	
	//ADCON1: 
		// bit 7-6: unimplemented			00	
		// bit 5-4: voltage ref config		00		Vdd-Vss
		// bit 3-0: A/D port config			1101	AN0, AN1 Analog; Else digital 
	adcon1 = 0b00001101;
	
	// AD0	PORTA	bit0
	// AD1	PORTA	bit1
	// AD2	PORTA	bit2
	trisa.0 = 1;
	trisa.1 = 1;
	
	//ADCON0:
		// bit 7-6: unimplemented						00
		// bit 5-2: analog channel select 				0000	AN0
		//												0001	AN1
		// bit 1: A/D GO/DONE (0=idle, 1=in prog)		0		
		// bit 0: A/D enable (0=disabled, 1=enabled)	0		Disabled
	adcon0 = (channel<<2) & 00111100b;
	
	//ADCON2:
		//bit 7: A/D result format				0		Left-Justified
		//bit 6: unimplemented					0
		//bit 5-3: A/D acquisition time			010		4 oscillations of the A/D clock = 6.4us > 2us (arbitratry)				
		//bit 2-0: A/D conversion select 		010		32*Tosc = 1.6us => See data sheet
	adcon2  = 0b00010010;

}




//////////////////////////////////////////
//	Function: ad_conv       
//	Purpose:  A/D conversion
//	Input:    Void                           
//  Output:   Unsigned short converted value
//////////////////////////////////////////
char ad_conv(){
	char store[30];
	char i;
	for(i=0;i<30;i++) {
		volatile bit godone @ADCON0.1;	// GO/DONE bit
		
		adcon0 |= 0b00000001; 			//turn on A/D module
		delay_us(10);					// Wait an aquisition time
		
		godone = 1;			 			// starts the conversion process
		while (godone);				 	// wait until conversion bit is idle
		
		store[i] = adresh;
		//unsigned short outl = adresl;
		//unsigned short out  = ((outh<<8) & (0b1111111100000000)) + outl; 
		
		adcon0 &= 0b11111110; 			//turn off A/D module
		}
	
	long sum = 0;
	for (i=0;i<30;i++) {
		sum = sum + store[i];
		}
	char out = sum/30;
	return out;
	}