#include <system.h>
#include "ZigBeelib.h"
#include "LCDlib_SPI.h"
#include "RTClib.h"
#include "ADlib.h"
#include "usartlib.h"
#include "EEPROMlib.h"
#include "motorlib.h"

#pragma DATA _CONFIG1H, _OSC_HS_1H 
#pragma DATA _CONFIG2H, _WDT_OFF_2H 
#pragma DATA _CONFIG4L, _LVP_OFF_4L & _XINST_OFF_4L
#pragma DATA _CONFIG3H, _MCLRE_ON_3H
#pragma CLOCK_FREQ 20000000

//////////////////////////////////////////
//	Smart Windows Project				//
//										//
//	SmartWindows.c						//
//  Main code for the project			//
//////////////////////////////////////////


//--// Communication Definitions
#define pc_attn    0
#define pc_poweron 1
#define win_max    10

//--// Function prototypes
	void PIC_sleep(void);			// Puts the Microchip in idle mode


//--// Voltatile bit Declarations
	
	// Timers
	volatile bit tmr0if@INTCON.2;		// Timer 0 Interrupt Flag
	
	// USART
	volatile bit rcif@PIE1.5;			// USART2 Receive Interrupt Flag
	
	// ZIGBEE
	volatile bit zb_rs @PORTD.1;			// ZigBee Reset
	volatile bit zb_slp@PORTD.2;			// ZigBee Sleep Signal
	volatile bit zb_irq@PORTD.3;			// ZigBee Interrupt Signal
	
	// Buttons
	volatile bit b0@PORTB.0;			// Button attached to B0
	volatile bit b1@PORTB.1;			// Button attached to B1
	volatile bit b2@PORTB.2;			// Button attached to B2
	volatile bit b3@PORTB.3;			// Button attached to B3
	volatile bit b4@PORTB.4;			// Button attached to B4
	volatile bit connect_button@PORTB.2;	// Connect button
	
	// Motor block
	volatile bit open_command  @PORTA.4;	// output
	volatile bit close_command @PORTA.5;	// output
	volatile bit close_button  @PORTB.1; 	// input
	volatile bit open_button   @PORTB.0;  	// input
	volatile bit open_stop     @PORTE.0;	// input
	volatile bit close_stop    @PORTE.1;	// input
	volatile bit middle_stop   @PORTE.2;	// input

	// RCU
	volatile bit RCU_open   @PORTB.0;
	volatile bit RCU_close  @PORTB.1;
	volatile bit RCU_up 	@PORTB.2;
	volatile bit RCU_down   @PORTB.3;

	
//--// Global Interrupt Semiphores
	bool tmr0is;						// Timer 0 Rollover Interrupt
	bool rcis;							// USART2 Character Receiver
	

//--// MAIN FUNCTION
void main(void){    
   	
   	//--// Setup I/O Pins
		adcon1 = 00001101b;
		intcon2.7 = 1;
		
		portb = 0;
		trisb.5 = 1;
		trisb.4 = 1;
		trisb.3 = 1;
		trisb.2 = 1;
		trisb.1 = 1;
	   	trisb.0 = 1;
	   
	   	trisa.5 = 0;
	   	trisa.4 = 0;
	   	
	   			
   	//-// General Variables
		unsigned short i;					// Loop variable used multiple places
		unsigned short j;					// Loop variable used multiple places
		unsigned short k;					// Loop variable used multiple places
		char debounce;
	
	//--// USART Initialization
		usart_init(57600);					// Initializes USART2 @ 57.6k baud rate
		char c = 'l';
	
	//--// Real Time Clock Initialization
		RTC_init();
		struct time  t;
		struct time* tp = &t;
		t.hr10  = 1;
		t.hr1   = 1;
		t.hr    = 11;
		t.min10 = 1;
		t.min1  = 1;
		t.min   = 11;
		t.sec10 = 1;
		t.sec1  = 1;
		t.sec   = 11;
		t.day   = 2;
		t.mon10 = 1;
		t.mon1  = 1;
		t.mon   = 11;
		t.date10= 1;
		t.date1 = 1;
		t.date  = 11;
		t.yr10  = 1;
		t.yr1   = 1;	
		t.yr    = 11;
		RTC_set(tp);
		delay_ms(100);
		t.day = 3;
		RTC_get(tp);
		RTC_print_terminal(tp);
		
		char alarm[8][5];
		for (i=0;i<8;i++){
			for (j=0;j<5;j++) {
				alarm[i][j] = 255;
				}
			}
		char check1 = 0;
		char check2 = 0;
		char check3 = 0;
		char check4 = 0;
		char days = 0;
		char sec_bcd=0;
		char min_bcd=0;
		char hr_bcd=0;
		usart_putc(c);
		
   	//-// ZigBee Initialization
		spi_init(0);						// Initialize the SPI
		usart_putc(c);	
		
		zb_init(11);						// Initialize for ZigBee
		usart_putc(c);	
		char seq = 0;						// Sequencing field for messages
		char success;						// Success of an operation

		char message[85];					// Holds messages to send
		unsigned short message_length;		// Holds length of current message
		char data[70];						// Holds data received
		unsigned short data_length;			// Holds length of current data
		char pc_instructions[win_max][11];	// Instruction from the PC to be transmitted
		char new_instructions[win_max];		// Indicates that new instrutions are available
		for(i=0;i<win_max;i++){				// Initialize to zero
			new_instructions[i] = 0;
			for(j=0;j<41;j++){
				pc_instructions[i][j]=0;
				}
			}
		char current_window = 254;			// Currently addressed window 254 - none, 255 - all
		
		char pan0;							// LS Byte of PAN address
		char pan1;							// MS Byte of PAN address
		char add0;							// LS Byte of device address
		char add1;							// MS Byte of device address
		char source_add1;					// Received message source
		char source_add0;					// Received message source
		
		unsigned short window_count;	// Number of connected windows
		char name[10][10];
		char status[10];
		char light[10];
		char battery[10];
		char disp[11];
		
	//--// USART Initialization
		//usart_init(57600);					// Initializes USART2 @ 57.6k baud rate
		//char c;
		
	//--// Interrupts Initialization
		intcon |= 11000000b;				// Global and Peripheral Interrupts enables
		tmr0is = 0;
		rcis = 0;
		//									// Enable speific interrupts seperatley

	//--// Type-dependant Code
		// CASE 0: PCU Base Station   -- Do not modify (D.C.S.)
		// CASE 1: OWU Window Station -- Do not modify (D.C.S)
		// CASE 2: RCU Remote Control -- Do not modify (D.C.S)
		// CASE 3: Product Testing and Evaluation
	switch (2) {
	
	//--// PCU 
		case 0:									// BASE STATION
			open_command = 0;
			close_command = 0;
			//--// ZigBee Addressing
			pan1 = 0xBA;
			pan0 = 0x5E;
			add1 = 0xA0;
			add0 = 0x00;
			zb_write_reg(PAN_ID_1,pan1);		// PAN ID 0xBA5E
			zb_write_reg(PAN_ID_0,pan0);
			zb_write_reg(SHORT_ADDR_1,add1);	// Device Address 0xA000
			zb_write_reg(SHORT_ADDR_0,add0);
			//--// Hold up to 10 Windows with name-length 10			
			window_count = EEPROM_read(0,0);// Get window count from EEPROM
			if (window_count==255) {
				window_count = 0;
				}
			for (i=0;i<window_count;i++){						// For each window...
				status[i]     =  EEPROM_read(0,11*(i+1)+0);		// ...Read in the status
				light[i] = 0;									// Default light reading
				for (j=0;j<10;j++){
					name[i][j] = EEPROM_read(0,11*(i+1)+1+j);	// ...Read in the name
					}
				}
	
			// ENABLE RECEIVER
			usart_receive_enable();
			
			while(1) {
				//---// INTERRUPT CHECK ROUTING
				if (tmr0is==1) {
					tmr0is=0;
					}
				if (rcis==1) {								// Received an attention signal
					rcis=0;									// Clear the semiphore
					c = usart_getc();
					delay_ms(400);
					usart_putc(c);
							
					//--// Startup Message
					if (c==254) {							// Startup Code
						for (i=0;i<window_count;i++){		// Loop through connect windows
							delay_ms(800);
							usart_putc(i);					// Put window number
							delay_ms(200);
							for (j=0;j<10;j++){
								usart_putc(name[i][j]);		// Put window name
								delay_ms(100);
								}
							}
						usart_putc(255);					// Put success code
						}
								
					//--// Regular Message
					else { // ELSE: Not a startup code
								
						//-// Obtain the rest of the instructions...
						current_window = c;				
								
						//-// ...for 255 signals
						if (c==255) {
							for(i=0;i<11;i++){pc_instructions[0][i]=usart_getc();}
							for(i=0;i<win_max;i++){
								for(j=0;j<11;j++){
									pc_instructions[i][j] = pc_instructions[0][j];
									}
								new_instructions[i] = 1;
								}
							
							//--// Special Cases
							switch (pc_instructions[0][0]) {
							case 'a':
								// NOT FILLING FOR SIMPLICITY (SHOULD BE '1','2','3','4')
							break;
							case 'n':
								// Record the names
								for(i=0;i<window_count;i++){
									for (j=0;j<10;j++){
										EEPROM_write(pc_instructions[i][j+1],0,11*(i+1)+1+j);
										}
									}
								for (i=0;i<window_count;i++){
									new_instructions[i] = 0;
									}									
							break;
							case 't':
								// Store time locally
								RTC_init();
								t.sec10 = (pc_instructions[0][1]>>4) & 00001111b; 
								t.sec1  =  pc_instructions[0][1]     & 00001111b;
								t.min10 = (pc_instructions[0][2]>>4) & 00001111b; 
								t.min1  =  pc_instructions[0][2]     & 00001111b;
								t.hr10  = (pc_instructions[0][3]>>4) & 00001111b; 
								t.hr1   =  pc_instructions[0][3]     & 00001111b;
								t.day   =  pc_instructions[0][4];
								RTC_set(tp);
								delay_ms(50);
								spi_init(0);
							break;
							case 'l':
								delay_s(1);
								for (i=0;i<window_count;i++) {
									if (light[i]==255){light[i]=254;}
									if (battery[i]==255){battery[i]=254;}
									}
								for (i=0;i<window_count;i++) {	// Put the readings
									usart_putc(light[i]);
									delay_ms(200);
									usart_putc(battery[i]);
									delay_ms(200);
									}
								usart_putc(255);				// Signal eng of message
								for (i=0;i<window_count;i++){
									new_instructions[i] = 0;	// Clear the new message
									}
							break;
							case 'e':
								for (i=0;i<window_count;i++){
									new_instructions[i] = 0;
									}
								window_count = 0;
								EEPROM_write(0,0,0);						// Erase Window count
								for (i=0;i<window_count;i++){				// For each window...
									EEPROM_write(0,0,11*(i+1)+0);			// ...overwrite in the status
									for (j=0;j<10;j++){
										EEPROM_write(0,0,11*(i+1)+1+j);		// ...overwrite in the name
										}
									}
							break;
							case 'c':
								// Nothing needed
							break;
							default:
								// Nothing needed
							break;
							}
							}
								
						//-// ... for individual windows
						else if (c<win_max) {
							for(i=0;i<11;i++){pc_instructions[current_window][i]=usart_getc();}
							new_instructions[current_window] = 1;
							
							//--// Special cases
							switch (pc_instructions[current_window][0]){
							case '1':
								new_instructions[current_window] = 2;
							break;
							case '2':
								new_instructions[current_window] = 2;
							break;
							case '3':
								new_instructions[current_window] = 2;
							break;
							case '4':
								new_instructions[current_window] = 2;
							break;
							case 'n':
								for (j=0;j<10;j++){
									EEPROM_write(pc_instructions[current_window][j+1],0,11*(i+1)+1+j);
									name[current_window][j] = pc_instructions[current_window][j+1];
									}
								new_instructions[current_window] = 0;
							break;
							default:
								// Nothing
							break;
							}
							}
						}
					usart_receive_enable();				// Reenable random communication
					}		
				//---// INTERRUPT CHECK ROUTINE
				
				
				success = 0;					// Reset the success flag to avoid mixups
				data[0] = 0xF0;					// Reset data flag to avoid mixups
				success = zb_rx(10000);			// Listen for a while
				
				if (success){
					message_length = zb_read_fb(message);				// Message Length
					data_length = message_length - header_length - 2;
					for (i=0; i<data_length; i++){						// Data extraction
						data[i] = message[header_length+i];
						}
					source_add0 = message[header_length-2];				// Source extraction
					source_add1 = message[header_length-1];

					// Use the data
					switch (source_add1) {
					//--// Remote request
						case 0xC0:						// Remote control unit
							if (data[0] == 0xFF) {		// Request for names
								message[0] = window_count;
								success = zb_tx(0xC0,0x00,message,1,seq);
								seq++;
								for (i=0;i<window_count;i++) {
									for (j=0;j<10;j++) {
										disp[j] = name[i][j];
										}
									success = 0;
									while (!success){
										success = zb_tx(0xC0,0x00,disp,10,seq);
										}
									seq++;
									}
								}
						break;
					//--// Window unit request
						default:							// On window unit
							
							//--// Request to join
							if (data[0] == 0x00) {
								
								window_count++;
								
								// Send the window its new address
								message[0] = 0x01;
								message[1] = pan1;
								message[2] = pan0;
								message[3] = 0xB0;
								message[4] = window_count-1;
								
								while (data[0] != 0x02) {
									seq = zb_broadcast(message,5,seq);
									success = zb_rx(30000);				// Listen for a while
									if (success){
										message_length = zb_read_fb(message);				// Message Length
										data_length = message_length - header_length - 2;
										for (i=0; i<data_length; i++){						// Data extraction
											data[i] = message[header_length+i];
											}
										source_add0 = message[header_length-2];				// Source extraction
										source_add1 = message[header_length-1];
										}					
									}
								name[window_count-1][0] = 'W';
								name[window_count-1][1] = 'i';
								name[window_count-1][2] = 'n';
								name[window_count-1][3] = 'd';
								name[window_count-1][4] = 'o';
								name[window_count-1][5] = 'w';
								name[window_count-1][6] = '0';
								name[window_count-1][7] = '0';
								name[window_count-1][8] = '0';
								name[window_count-1][9] = window_count-1 +'0';
								
								// Update the records
								EEPROM_write(window_count,0,0);
								i=window_count-1;
								EEPROM_write(window_count-1,0,11*(i+1)+0);
								for (j=0;j<10;j++){
									EEPROM_write(name[window_count-1][j],0,11*(i+1)+1+j);
									}																
								
								// Clean the new_instructions
								new_instructions[window_count-1] = 0;
								}
							
							//--// Request for instructions
							else if (data[0] == 0x03) {		
								light[source_add0] = data[1];		// Second byte is the light data
								battery[source_add0] = data[2];		// Third byte is battery data
								
								// New Instructions available for correct window
								if (new_instructions[source_add0]){
									for (i=0;i<11;i++){
										message[i] = pc_instructions[source_add0][i];
										}
									success = zb_tx(source_add1,source_add0,message,11,seq);
									seq++;
									if (new_instructions[source_add0]==2){
										usart_putc('a');
										}
									new_instructions[source_add0] = 0;
									}
								
								// No new Instructions available for the correct window
								else{
									message[0] = 0x00;
									success = zb_tx(source_add1,source_add0,message,1,seq);
									seq++;
									}
								}
						break;
						}
					}
				if (rcsta.1){
					usart_receive_disable();			// Clear any reception errors
					usart_receive_enable();
					}
				}
			// DISABLE RECEIVER
			usart_receive_disable();
		break;
			
	//--// OWU
		case 1:									// REMOTE STATION
			// OWU Variables
			motor_init();
			char mode = 0;						// 0 (1,2) manual, 3 green, 4 security 
			char state;							// 0 closed, 1 middle, 2 open		
				if (porte.0){state=2;}
				else if (porte.1){state=0;}
				else {state=1;}
			
			// ZigBee addressing
			pan1 = EEPROM_read(0x00,0x00);		// PAN ID from EEPROM
			pan0 = EEPROM_read(0x00,0x01);
			add1 = EEPROM_read(0x00,0x02);		// Device Address from EEPROM
			add0 = EEPROM_read(0x00,0x03);
			zb_write_reg(PAN_ID_1,pan1);		
			zb_write_reg(PAN_ID_0,pan0);
			zb_write_reg(SHORT_ADDR_1,add1);	
			zb_write_reg(SHORT_ADDR_0,add0);	
			
			// Initialize Timer1
			t0con = 10000111b;					// Initialize T0
			tmr0h = 46005/256;					// Set T0 to interrupt @ 1 Hz
			tmr0l = 46005%256;
			
			// Initialize adcon for light sensor
			ad_init(0);
			char light_reading = 0;
			char bat_reading = 0;
			
			// Technical Readout
			usart_printf("\n\rTechnical Read Out:\n\rPower On");
			
			while(1) {
				//---// INTERRUPT CHECK ROUTING
					if (tmr0is==1) {
						tmr0is=0;
						}
					if (rcis==1) {
						rcis=0;
						usart_receive_enable();
						}
				
				// Technical Readout
				usart_printf("\n\r>");
				
				//--// Check the light-sensor and battery
					light_reading = ad_conv();
					ad_init(1);
					delay_ms(50);
					usart_printf("{L:");
					usart_putShort(light_reading);
					usart_printf(" B:");
					bat_reading = ad_conv();
					usart_putShort(bat_reading);
					usart_printf("} ");
					ad_init(0);
					
				//--// STATE MACHINE	
					switch (mode) {
					case 3:		// Green Mode
						//Check Buttons
						if(open_button||close_button){
							debounce = 0;
							for (i=0;i<50;i++){
								if (open_button||close_button) {debounce++;}
								}
							if (debounce>25) {mode = 0;}
							}
						while(open_button  && (!open_stop )) {open_command  = 1;}
						open_command  = 0;
						while(close_button && (!close_stop)) {close_command = 1;}
						close_command = 0;
						//Check Light
						if (porte.0){state=2;}		// Open
						else if (porte.1){state=0;}	// Close
						else {state=1;}				// Middle
						switch (state) {
							case 0:
								if (light_reading < 75) {motor_open();}
							break;
							case 2:
								if (light_reading >= 75) {motor_close();}
							break;
							default:
								if (light_reading < 75) {motor_open();}
								else                     {motor_close();}
							break;
							}
					break;
					case 4:		// Security Mode
						//Check Buttons
						if(open_button||close_button){
							debounce = 0;
							for (i=0;i<100;i++){
								if (open_button||close_button) {debounce++;}
								}
							if (debounce>50) {mode = 0;}
							}
						while(open_button  && (!open_stop )) {open_command  = 1;}
						open_command  = 0;
						while(close_button && (!close_stop)) {close_command = 1;}
						close_command = 0;
						//Check Alarms
						RTC_init();
						RTC_get(tp);
						/*
						//RTC_print_terminal(tp);
						sec_bcd = (t.sec10<<4) | t.sec1;						
						min_bcd = (t.min10<<4) | t.min1;	
						hr_bcd  = (t.hr10<<4)  | t.hr1;
						usart_putc(' ');
						usart_putByte(sec_bcd);
						usart_putc(':');
						usart_putByte(min_bcd);
						usart_putc(':');
						usart_putByte(hr_bcd);
						usart_putc(':');
						usart_putShort(t.day);
						usart_putc(' ');
						*/	
						for (i=0;i<8;i++) {
							check1 = 0;
							check2 = 0;
							check3 = 0;
							check4 = 0;
							if (alarm[i][1] != 255){
								usart_printf("{A");
								usart_putShort(i);
								usart_printf(":");
								if (alarm[i][1] == sec_bcd) {
									check1=1;
									usart_putc('1');
									}
								if (alarm[i][2] == min_bcd) {
									check2=1;
									usart_putc('2');
									}
								if (alarm[i][3] == hr_bcd) {
									check3=1;
									usart_putc('3');
									}
								days = alarm[i][4];
								switch (t.day){
									case 1: 
										check4 = days.0; 
										if (check4) {usart_putc('4');}
										break;
									case 2: 
										check4 = days.1; 
										if (check4) {usart_putc('4');}
										break;
									case 3: 
										check4 = days.2; 
										if (check4) {usart_putc('4');}
										break;
									case 4: 
										check4 = days.3; 
										if (check4) {usart_putc('4');}
										break;
									case 5: 
										check4 = days.4; 
										if (check4) {usart_putc('4');}
										break;
									case 6: 
										check4 = days.5; 
										if (check4) {usart_putc('4');}
										break;
									case 7: 
										check4 = days.6; 
										if (check4) {usart_putc('4');}
										break;
									default:
										break;
									}
								if (check1 && check2 && check3 && check4){
									switch (alarm[i][0]) {
										case 0:
											motor_open();
										break;
										case 1:
											motor_middle();
										break;
										case 2:
											motor_close();
										break;
										default:
										break;
										}
									}
								usart_printf("}");
								}
							}	
						spi_init(0);	// Initialize the SPI
					break;
					default:	// Manual Mode	
						if (middle_stop) {usart_putc('m');}
						while(open_button  && (!open_stop )) {open_command  = 1;}
						open_command  = 0;
						while(close_button && (!close_stop)) {close_command = 1;}
						close_command = 0;
					break;
					}
				
				//--// Wake up the ZigBee
					zb_wake();

				//--// Contact the PC
					message[0] = 0x03;
					message[1] = light_reading;
					message[2] = bat_reading;
					success = zb_tx(0xA0,0x00,message,3,seq);
					seq++;
					success = 0;
					success = zb_rx(2000);				 		// Listen for a message
					if (success){
						message_length = zb_read_fb(message);	// Once one is received, read FB
						// Extract the data
						data_length = message_length - header_length - 2;
						data[0] = message[header_length];
						usart_printf(" {M: ");
						for (i=0; i<data_length; i++){
							data[i] = message[header_length+i];
							}
						usart_putc(data[0]);
						usart_printf("}");
						switch(data[0]) {
							case 'c':
								switch (data[1]) {
									case 0:
										mode = 0;
										motor_open();
									break;
									case 1:
										mode = 0;
										motor_middle();
									break;
									case 2:
										mode = 0;
										motor_close();
									break;
									case 3:
										mode = 3;
									break;
									case 4:
										mode = 4;
									break;
									}
							break;
							case '1':
								k = 1;
								for (i=0;i<2;i++) {
									for (j=0;j<5;j++) {
										alarm[i][j] = data[k];
										k++;
										}
									}	
							break;
							case '2':
								k = 1;
								for (i=2;i<4;i++) {
									for (j=0;j<5;j++) {
										alarm[i][j] = data[k];
										k++;
										}
									}									
							break;
							case '3':
								k = 1;
								for (i=4;i<6;i++) {
									for (j=0;j<5;j++) {
										alarm[i][j] = data[k];
										k++;
										}
									}									
							break;
							case '4':
								k = 1;
								for (i=6;i<8;i++) {
									for (j=0;j<5;j++) {
										alarm[i][j] = data[k];
										k++;
										}
									}									
								/*
								usart_printf("\n\r\n\rAlarm1: ");
								usart_putByte(alarm[0][0]);
								usart_putc(' ');
								usart_putByte(alarm[0][1]);
								usart_putc(' ');
								usart_putByte(alarm[0][2]);
								usart_putc(' ');
								usart_putByte(alarm[0][3]);
								usart_putc(' ');
								usart_putByte(alarm[0][4]);
								usart_printf("\n\r\n\rAlarm2: ");
								usart_putByte(alarm[1][0]);
								usart_putc(' ');
								usart_putByte(alarm[1][1]);
								usart_putc(' ');
								usart_putByte(alarm[1][2]);
								usart_putc(' ');
								usart_putByte(alarm[1][3]);
								usart_putc(' ');
								usart_putByte(alarm[1][4]);
								usart_printf("\n\r\n\rAlarm3: ");
								usart_putByte(alarm[2][0]);
								usart_putc(' ');
								usart_putByte(alarm[2][1]);
								usart_putc(' ');
								usart_putByte(alarm[2][2]);
								usart_putc(' ');
								usart_putByte(alarm[2][3]);
								usart_putc(' ');
								usart_putByte(alarm[2][4]);
								*/
							break;
							case 't':
								RTC_init();
								t.sec10 = (data[1]>>4) & 00001111b; 
								t.sec1  =  data[1]     & 00001111b;
								t.min10 = (data[2]>>4) & 00001111b; 
								t.min1  =  data[2]     & 00001111b;
								t.hr10  = (data[3]>>4) & 00001111b; 
								t.hr1   =  data[3]     & 00001111b;
								t.day   =  data[4];
								RTC_set(tp);
								delay_ms(10);
								spi_init(0);	// Initialize the SPI
								//zb_init(11);	// Initialize for ZigBee
							break;
							case 'e':
								pan1 = 0x00;
								pan0 = 0x00;
								add1 = 0x00;
								add0 = 0x00;
								zb_write_reg(PAN_ID_1,pan1);		// PAN ID from EEPROM
								zb_write_reg(PAN_ID_0,pan0);
								zb_write_reg(SHORT_ADDR_1,add1);	// Device Address from EEPROM
								zb_write_reg(SHORT_ADDR_0,add0);	
								EEPROM_write(pan1,0x00,0x00);
								EEPROM_write(pan0,0x00,0x01);
								EEPROM_write(add1,0x00,0x02);
								EEPROM_write(add0,0x00,0x03);
							break;
							default:
							break;
							}							
						}

				//--// Contact the Remote
					message[0] = 0x03;
					success = zb_tx(0xC0,0x00,message,1,seq);
					seq++;
					success = 0;
					success = zb_rx(1000);				 		// Listen for a message
					if (success){
						message_length = zb_read_fb(message);	// Once one is received, read FB
						// Extract the data
						data_length = message_length - header_length - 2;
						data[0] = message[header_length];
						for (i=0; i<data_length; i++){
							data[i] = message[header_length+i];
							}
						if (porte.0){state=2;}		// Open
						else if (porte.1){state=0;}	// Close
						else {state=1;}				// Middle
						switch (data[0]) {
							case 0x01:
								if 		(state==0) {motor_open();}
								else if (state==1) {motor_open();  }
							break;
							case 0x02:
								if      (state==2) {motor_middle();}
								else if (state==1) {motor_close();}
							break;
							}
						}
				
				//--// CONNECT TO BASE STATION
				if (connect_button){
					// Send broadcast message with request/panid/address
					message[0] = 0x00;
					message[1] = pan1;
					message[2] = pan0;
					message[3] = add1;
					message[4] = add0;
					seq = zb_broadcast(message,5,seq);

					// Listen for response with new panid/address
					data[0]=0;
					while(data[0] != 0x01){
						success = 0;
						success = zb_rx(0);
						if (success){
							message_length = zb_read_fb(message);
							// Extract the data
							data_length = message_length - header_length - 2;
							for (i=0; i<data_length; i++){
								data[i] = message[header_length+i];
								}
							}
						}
					// Use the data
					zb_write_reg(PAN_ID_1,data[1]);		// PAN ID from EEPROM
					zb_write_reg(PAN_ID_0,data[2]);
					zb_write_reg(SHORT_ADDR_1,data[3]);	// Device Address from EEPROM
					zb_write_reg(SHORT_ADDR_0,data[4]);	
					EEPROM_write(data[1],0x00,0x00);
					EEPROM_write(data[2],0x00,0x01);
					EEPROM_write(data[3],0x00,0x02);
					EEPROM_write(data[4],0x00,0x03);
					// Send connected message
					message[0] = 0x02;
					success = zb_tx(0xA0,00,message,1,seq);
					seq++;
					// Tell Technical Readout
					usart_printf("{Connected as # ");
					usart_putShort(data[4]);
					usart_printf("} ");
					delay_s(5);
					}
				
				//--// Sleep cycle
					zb_sleep();
					PIC_sleep();
				}
		break;
		
	//--// RCU
		case 2:
			//--// Timer0 Init
				t0con = 10000111b;					// Initialize T0
				intcon |= 00100000b;				// Enable the T0 interrupt to occur @ tmr0if
				tmr0h = 46005/256;					// Set T0 to interrupt @ 1 Hz
				tmr0l = 46005%256;
				char tmr0count = 0;					// Reset the timer interrupt counter
				char sleep_timer = 0;				// Fall asleep
				
			//--// Set Device Addressing
				pan1 = 0xBA;	
				pan0 = 0x5E;
				add1 = 0xC0;
				add0 = 0x00;
				zb_write_reg(PAN_ID_1,pan1);		// PAN ID 0xBA5E
				zb_write_reg(PAN_ID_0,pan0);
				zb_write_reg(SHORT_ADDR_1,add1);	// Device Address 0xC000
				zb_write_reg(SHORT_ADDR_0,add0);
			
			//--// Prepare to accept Current Windows from Base Station
				window_count=1;					// Number of connected windows
				current_window = 0;				// Currently addressed window
				for (i=0;i<10;i++){
					name[i][0] = 'E';
					name[i][1] = 'm';
					name[i][2] = 'p';
					name[i][3] = 't';
					name[i][4] = 'y';
					name[i][5] = ' ';
					name[i][6] = ' ';
					name[i][7] = ' ';
					name[i][8] = ' ';
					name[i][9] = ' ';		
					status[i] = 0;
					}
			//--// Request up to date window information
				data[0] = 0xFF;
				success = zb_tx(0xA0,0x00,data,1,seq);			// Are you there
				seq++;
				if (success) {									// If a head station responds
					source_add1 = 0;
					while (source_add1 != 0xA0) {				// Listen for return byte from HEAD
						success = zb_rx(0);							
						message_length = zb_read_fb(message);
						source_add1 = message[header_length-1];	// Source extraction					
						}
					for (i=0; i<message_length; i++){			// Data extraction
						data[i] = message[header_length+i];
						}
					window_count = data[0]; 					// Returned byte is the window count
		
					for (i=0;i<window_count;i++) {				// Receive the window names
						source_add1 = 0;
						while (source_add1 != 0xA0) {			// Listen for byte from HEAD
							success = zb_rx(0);
							message_length = zb_read_fb(message);
							source_add1 = message[header_length-1];	// Source extraction					
							}
						for (j=0; j<message_length; j++){			// Data extraction
							data[j] = message[header_length+j];
							name[i][j] = data[j]; 					// byte string is the window name
							}
						name[i][message_length] = '\0';				// End of string character
						}
					name[window_count][0] = 'S';
					name[window_count][1] = 'o';
					name[window_count][2] = 'l';
					name[window_count][3] = 'a';
					name[window_count][4] = 'r';
					name[window_count][5] = 'S';
					name[window_count][6] = 'h';
					name[window_count][7] = 'a';
					name[window_count][8] = 'd';
					name[window_count][9] = 'e';
					name[window_count][10] = '\0';
					}

			//--// Setup the LCD
				SPILCD_init();
				delay_ms(100);
				SPILCD_brightness(6);		
				delay_ms(100);
				SPILCD_clear();
				delay_ms(100);
				SPILCD_putShort(window_count);
				SPILCD_printf(" windows.");
				delay_ms(100);
				SPILCD_home();
				delay_ms(100);
				spi_init(0);
				delay_ms(2);
				
			
			//--// Main Loop		
			while(1) {
				//---// INTERRUPT CHECK ROUTINE
						if (tmr0is==1) {
							tmr0is=0;
							tmr0count++;
							sleep_timer++;
							if (sleep_timer>180){
								SPILCD_init();
								SPILCD_brightness(0);
								spi_init(0);
								delay_ms(100);
								zb_sleep();
								while(!(RCU_up||RCU_down||RCU_open||RCU_close));
								zb_wake();
								delay_ms(100);
								SPILCD_init();
								delay_ms(100);
								SPILCD_brightness(6);
								delay_ms(100);
								SPILCD_clear();
								spi_init(0);
								sleep_timer=0;
								}
							if (tmr0count>60) {
								tmr0count = 0;
								// Get current windows again
								data[0] = 0xFF;
								success = zb_tx(0xA0,0x00,data,1,seq);			// Are you there
								seq++;
								if (success) {									// If a head station responds
									source_add1 = 0;
									while (source_add1 != 0xA0) {				// Listen for return byte from HEAD
										success = zb_rx(0);
										message_length = zb_read_fb(message);
										source_add1 = message[header_length-1];	// Source extraction					
										}
									for (i=0; i<message_length; i++){			// Data extraction
										data[i] = message[header_length+i];
										}
									window_count = data[0]; 					// Returned byte is the window count
									for (i=0;i<window_count;i++) {				// Receive the window names
										source_add1 = 0;
										while (source_add1 != 0xA0) {			// Listen for byte from HEAD
											success = zb_rx(0);
											message_length = zb_read_fb(message);
											source_add1 = message[header_length-1];	// Source extraction					
											}
										for (j=0; j<message_length; j++){			// Data extraction
											data[j] = message[header_length+j];
											name[i][j] = data[j]; 					// byte string is the window name
											}
										name[i][message_length] = '\0';				// End of string character
										}
										name[window_count][0] = 'S';
										name[window_count][1] = 'o';
										name[window_count][2] = 'l';
										name[window_count][3] = 'a';
										name[window_count][4] = 'r';
										name[window_count][5] = 'S';
										name[window_count][6] = 'h';
										name[window_count][7] = 'a';
										name[window_count][8] = 'd';
										name[window_count][9] = 'e';
										name[window_count][10] = '\0';
									}
								}
							}
				//---// INTERRUPT CHECK ROUTINE
				
				//--// Check the Buttons
				if (RCU_up) {
					current_window++;
					if (current_window>window_count) {current_window = 0;}
					sleep_timer = 0;
					}
				else if (RCU_down) {
					if (current_window==0) {current_window = window_count;}
					else {current_window--;}
					sleep_timer = 0;
					}
				else if (RCU_open) {
					status[current_window] = 1;
					sleep_timer = 0;
					}
				else if (RCU_close) {
					status[current_window] = 2;
					sleep_timer = 0;
					}
				
				//--// Pass along the most recent command	
				success = 0;					// Reset the success flag to avoid mixups
				success = zb_rx(1000);			// Listen for a while
				if (success) {
					message_length = zb_read_fb(message);
					source_add0 = message[header_length-2];	 // Source extraction
					source_add1 = message[header_length-1];						
					if ((source_add1==0xB0)&&(status[source_add0])){
						data[0] = status[source_add0];
						status[source_add0] = 0;
						success = zb_tx(source_add1,source_add0,data,1,seq);
						seq++;
						}
					else if ((source_add1==0xD0)&&(status[window_count])) { // Solar Shade
						data[0] = status[window_count];
						status[window_count]=0;
						while(RCU_open||RCU_close) {
							success = 0;
							success = zb_rx(1000);
							if (success) {
								zb_tx(0xD0,0x00,data,1,seq); // 1 open, 2 close
								seq++;
								}
							}
						success = 0;
						i = 0;
						while((!success)&&(i<100)){success = zb_rx(1000);}
						data[0] = 0;
						for (i=0;i<10;i++) {
							success = zb_tx(0xD0,0x00,data,1,seq); // 0 stop
							seq++;
							}
						}
					}						
				
				SPILCD_init();
				delay_ms(50);
				SPILCD_clear();
				delay_ms(35);
				for (i=0;i<10;i++){
					disp[i] = name[current_window][i];
					}
				disp[10] = '\0';
				SPILCD_printf(disp);
				delay_ms(10);
				spi_init(0);
				delay_ms(50 );
				}
				
		break;

	//--// Product Testing
		case 3:
			char thecount = 0+'a';
			while (1) {
				usart_putc(thecount);
				thecount++;
				delay_s(1);
				if (thecount>10+'a') {
					thecount = 0+'a';
					}
				}
		break;
		
	//--// Default Mode
		default:
			SPILCD_init();
			while(1){
				//SPILCD_brightness(5);		
				//SPILCD_clear();
				SPILCD_printf("Catch");
				//SPILCD_putShort(window_count);
				//spi_init(0);
				}
		break;
		}
	}
	
	
	
///////////////////////////////////////////
// 	Function: interrupt
//	Purpose:  The interrupt service routine
//	Input:	  Void
//  Output:   Void
///////////////////////////////////////////
void interrupt(void) {
	
	///// IMPORTANT //////////////////////////////////////////////////
	// Semiphores Needed: 	tmr0is --> timer0 rollover interrupt
	// Ensure that these semiphores are globally defined before using
	//////////////////////////////////////////////////////////////////
	
	if(tmr0if != 0) {			// Overflow on timer0
		tmr0is = true; 			// Set the semiphore
		tmr0if = 0; 			// Reset the flag
		tmr0h = 46005/256;		// Reset the timer0 counter to 46005
		tmr0l = 46005%256;			
		}
	if(rcif != 0) {				// USART Received Character
		char c;
		c = usart_foundc();	// Reset the flag by reading the buffer
		if (c==pc_attn) {		// If the message signal has been given
			rcis = true;		// Set the semiphore
			usart_receive_disable();
			delay_ms(400);
			usart_putc(0);
			}
		c = '\0';				// Reset c
		}
		
	return;
	}



////////////////////////////////////////////////////////////
// Function: sleep_PIC
// Purpose:  puts the microcontroller into idle_primary mode
// Inputs:   Void
// Outputs:  Void
////////////////////////////////////////////////////////////
void PIC_sleep(void){
	intcon |= 00100000b;				// Enable the T0 interrupt to occur @ tmr0if
	// Set for idle mode on sleep commmand
	tmr0h = 46005/256;					// Set T0 to interrupt @ 1 Hz
	tmr0l = 46005%256;
	
	osccon |= 10000000b;
	// ASSUMING SCS = 00 (DEFAULT)
	
	// Execute sleep command
	nop();
	nop();
	nop();
	sleep();
	
	// Wake on interrupt
	nop();
	nop();
	nop();
	intcon &= 11011111b;				// Disable T0 interrupt
	return;
	}
	