#ifndef _ZIGBEELIB_H_
#define _ZIGBEELIB_H_
#include<system.h>

//////////////////////////////////////////
//	Smart Windows Project				//
//										//
//	ZigBeelib.h							//
//   	SPI function headers for ZigBee	//
//////////////////////////////////////////


// Constants
#define header_length	12			// Length of the message before data

// Structures
struct window {
	char status;			// Window status
	char name[16];			// Name of the window
	};

// Registers
#define	TRX_STATUS 		0x01
#define	TRX_STATE 		0x02
#define	TRX_CTRL_0 		0x03
#define	TRX_CTRL_1		0x04
#define	PHY_TX_PWR		0x05
#define	PHY_RSSI		0x06
#define	PHY_ED_LEVEL	0x07	
#define	PHY_CC_CCA		0x08
#define	CCA_THRES		0x09
#define	RX_CTRL			0x0A
#define	SFD_VALUE		0x0B
#define	TRX_CTRL_2		0x0C
#define	ANT_DIV			0x0D
#define	IRQ_MASK		0x0E     	// Set to enable an interrupt
#define	IRQ_STATUS		0x0F		// Check to see if interrupt happened
#define	VREG_CTRL		0x10
#define	BATMON			0x11
#define	XOSC_CTRL		0x12
#define SHORT_ADDR_0	0x20
#define SHORT_ADDR_1	0x21
#define PAN_ID_0		0x22
#define PAN_ID_1		0x23


// States
#define	P_ON 					0x00
#define	BUSY_RX					0x01
#define	BUSY_TX 				0x02
#define	RX_ON					0x06
#define	TRX_OFF					0x08
#define	PLL_ON					0x09
#define	SLEEP	0x0F	
#define	BUSY_RX_AACK			0x11
#define	BUSY_TX_ARET			0x12
#define	RX_AACK_ON				0x16
#define	TX_ARET_ON				0x19
#define	RX_ON_NOCLK				0x1C
#define	RX_AACK_ON_NOCLOCK		0x1D
#define	BUSY_RX_AACK_NOCLK		0x1E
#define	STATE_TX_IN_PROGRESS	0x1F

// State Commands
#define	NOP 				0x00
#define	TX_START	 		0x02
#define FORCE_TX_OFF		0x03
#define FORCE_PLL_ON		0x04
//#define	RX_ON				0x06
//#define	TRX_OFF				0x08
//#define	PLL_ON				0x09
//#define	RX_AACK_ON			0x16
//#define	TX_ARET_ON			0x19


// ZigBee SPI1 Function Prototypes
// See ZigBeelib.c for actual functions
void spi_init (unsigned short rate);									// Initializes the SPI
char spi_byte(char value);		       									// Sends and receives a byte by SPI
void zb_init(char channel);												// Initializes the Atmel ZigBee card
char zb_read_reg(char add);												// Reads a register on ZigBee card
void zb_write_reg(char add, char data);									// Writes a regiser on the ZigBee card
char zb_read_fb(char* tmp);												// Reads the framebuffer on the ZigBee card
char zb_write_fb(char add1, char add0,char data[],char size,char seq);	// Writes to the framebuffer on the ZigBee card
void zb_state_trans(char state, char command);							// Causes the ZigBee card to change states
char zb_tx (char add1, char add0, char data[], char size, char seq);	// Transmits a packet over ZigBee
char zb_rx (unsigned short length);										// Receives a data packet
void zb_sleep(void);													// Puts the ZigBee chip to sleep
void zb_wake(void);														// Wakes the ZigBee chip from sleep
char zb_broadcast_fb(char data[],char size,char seq);					// Prepares a broadcast paket for ZigBee
char zb_broadcast(char data[], char size, char seq);					// Transmits a broadcast packet over ZigBee

#endif //_ZIGBEELIB_H_