#include <system.h>
#include "ZigBeelib.h"
#include "LCDlib_SPI.h"
#include "RTClib.h"
#include "ADlib.h"
#include "usartlib.h"
#include "EEPROMlib.h"
#include "motorlib.h"

//////////////////////////////////////////
//	Smart Windows Project				//
//										//
//	usartlib.c							//
//  Serial functions for usart			//
//////////////////////////////////////////

// Function Prototypes are in usartlib.h

// Global Volatile Bits

////////////////////////////////////////////////////////
//	Function: usart_init
//	Purpose:  initializes usart
//	Input:    rate -> Must be 57600 for stable operation
//  Output:   Void
////////////////////////////////////////////////////////
void usart_init(unsigned short rate)
{
	trisc  |= 11000000b;		// Set TX and Receive pins for eusart 
	baudcon = 00000000b; 		// 8-bit baud rate
		
	if (rate == 57600)
		spbrg = 21;	            // Set correct spbrg2 correctly For 57.6k rate
	else
		spbrg = (1250000/rate)-1;  // Otherwise, attempt to find the "n value" as best as possible
								   // THIS CODE HAS AN OVERFLOW ERROR -> DON'T USE
	
	/**/	//rcsta2 default is 00000000b
			//rcsta2 = 10000000b;
			//bit 7: enable serial port - 1
			//bit 6: 8 bit reception    - 0
			//bit 5: N/A                - x
			//bit 4: enable receiver    - 0
			//bit 3: N/A                - x
			//bit 2: No framing error.  - x
			//bit 1: No overrun error.  - x
			//bit 0: N/A                - x
	rcsta = 10000000b; 			// Enable serial port
		
	/**/	//trxsta default is 00000010b
			//txsta = 00100100b;		
			//bit 7: x
			//bit 6: 8-bit transmit => 0
			//bit 5: transmit enable => 1
			//bit 4: asynch => 0
			//bit 3: not "sync break on transmission completed" => 0
			//bit 2: high speed => 1
			//bit 1: "TSRx empty" => x
			//bit 0: Parity bit => x
	txsta = 00100100b; 			// High speed baud rate mode
}



////////////////////////////////////////////////////
//	Function: usart_putc
//	Purpose:  puts a character to terminal via usart
//  Input:    c -> character to be typed (ascii)
//  Output:   Void
////////////////////////////////////////////////////
void usart_putc(char c)
{
	volatile bit transmitReady@PIR1.4;	// transmission signal bit
	while(!transmitReady);				// Wait until usart is ready (bit goes high)
	txreg = c;							// Put the data to be written into the Register
}



///////////////////////////////////////////////////////
//	Function: usart_getc
//	Purpose:  gets a characeter from terminal via usart
//	Input:	  Void
//  Output:   The character captured from the terminal
///////////////////////////////////////////////////////
char usart_getc(void)
{
	char value;							// Allocate memory For a character
	volatile bit receiveReady@PIR1.5;	// Monitor receive signal flag
	unsigned long ii = 0;
	char error = 0;
	rcsta |= 00010000b;					// Enable the receiver	
	while(!receiveReady){				// Wait until usart2 is ready (bit goes high)
		if (ii>750000) {
			error = 1;
			break;
			}
		ii++;
		}							
	value = rcreg;						// Get that value in the buffer
	rcsta &= 11101111b;					// Disable the receiver to prevent overflow
	if (error) {value = 0;}
	return value;						// Return the character
}



////////////////////////////////////////////////////////
//	Function: usart_receive_enable
//	Purpose:  Enables the receiver for character finding
//	Input:	  Void
//  Output:   Void
//////////////////////////////////////////////////////////
void usart_receive_enable(void) {
	pie1 |= 00100000b;				// Enable receiver interupts
	rcsta |= 00010000b;				// Enable the receiver	
	return;	
	}


////////////////////////////////////////////////////////
//	Function: usart_receive_disable
//	Purpose:  Enables the receiver for character finding
//	Input:	  Void
//  Output:   Void
//////////////////////////////////////////////////////////
void usart_receive_disable(void) {
	rcsta &= 11101111b;					// Disable the receiver to prevent overflow
	pie1 &= 11011111b;					// Disable the receiver interrupts
	return;	
	}
	
	
	
/////////////////////////////////////////////////////////////////
// Funtion:	usart_foundC
// Purpose:	Interrupt driven mechanism for finding cs on terminal
// Input:	Void
// Output:	The character captured from the terminal
/////////////////////////////////////////////////////////////////
char usart_foundc(void)
{
	char value = rcreg;
	return value;
}



///////////////////////////////////////////////////////
//	Function: usart_printf
//	Purpose:  prints a character string to the terminal
//	Input:	  c[] -> variable length character array
// 	Output:   Void
///////////////////////////////////////////////////////
void usart_printf(char c[])
{
	for(int i=0; c[i] != '\0'; i++)		// Loop through each character in the array
		usart_putc(c[i]);				// Put the character to the terminal
}



//////////////////////////////////////////////////////////////////////
//	Function: usart_putShort
//	Purpose:  prints an unsigned short to the terminal in decimal form
//	Input:    dec -> that value to be printed
//	Output:   Void
//////////////////////////////////////////////////////////////////////
void usart_putShort(unsigned short dec)
{
	unsigned short dec1;		// Ten Thousands place
	unsigned short dec2;		// Thousands place
	unsigned short dec3;		// Hundreds place
	unsigned short dec4;		// Tens place
	unsigned short dec5;		// Ones place
	
	dec1 = dec/10000;			// Get the Ten Thousands place with division
	dec = dec%10000;			// Calculate how much is left
	dec2 = dec/1000;			// Get the Thousands place with division
	dec = dec%1000;				// Calculate how much is left
	dec3 = dec/100;				// Get the Hundreds place with division
	dec = dec%100;				// Calculate how much is left
	dec4 = dec/10;				// Get the Tens place with division
	dec5 = dec%10;				// The Ones place is how much is left
	
	if(dec1 != 0)				// If there is a Ten Thousands place, print it
		usart_putc(dec1 + '0');
	if(dec2 != 0)				// If there is a Thousands place, print it
		usart_putc(dec2 + '0');
	if(dec3 != 0)				// If there is a Hundreds place, print it
		usart_putc(dec3 + '0');
	if(dec4 != 0)				// If there is a Tens place, print it
		usart_putc(dec4 + '0');
	usart_putc(dec5 + '0');	// Print the one's place
}



////////////////////////////////////////////////////////////
//	Function: usart_getShort
//	Purpose:  reads in an decimal number from the terminal
//  Input:    Void
//  Output:   The decimal input in unsigned short format
////////////////////////////////////////////////////////////
unsigned short usart_getShort(void)
{
	char	userInput;			// User's input as a character
	int	userInt;				// User's current input converted to integer
	unsigned int add = 0;		// The user's whole input as an integer
	unsigned short short_add;	// The user's whole input as a unsign. Short
	int i;						// Loop variable
	
	usart_printf("Please enter your digits:");		// Requests a decimal number from user
	for(i=0; i<5; i++)								// Get a digit a max of 5 times (For an unsign. Short)
	{
		userInput = usart_getc();					// Get the user's next digit
		if((userInput=='\r')||(userInput=='\n')){	// If the digit is a "return" or "new line," Break from the loop
			break;
		}
		else if((userInput>='0')&&(userInput<='9')){// If the user has added a valid digit
			userInt = userInput - '0';				// Save the current digit in interger form
		}
		else {										// If the user did not enter a valid digit
			usart_printf("That is not a valid digit.");// Print an error message
			return 0;								// Return a zero
		}
		add = add*10 + userInt;						// Add the new digit to old total (weighed appropriately) 
	}			
	
	if(add < 32768)									// If the number is valid
		short_add = add;							// Store it as a Short
	else 	{										// If the number is too large
		usart_printf("Too large for a short.");	// Print an error
		short_add = add;							// Save it (truncated) as a Short
		}
	
	return short_add;								// Return the user's Short
}



////////////////////////////////////////////
//	Function: usart_getByte
//	Purpose:  reads in a byte in binary form
//  Input:    Void
//  Output:   The input byte in char format
////////////////////////////////////////////
char usart_getByte(void) {
	char out_byte = 10000000b;		// User's input as a character
	char input;						// User's current input character
	int i;							// Loop variable
	
	for(i=0; i<8; i++) {			// Get a digit a max of 5 times (For an unsign. Short
		input = usart_getc();		// Get the user's next character
		
		if (input == '1') {			// Character is a 1
			out_byte = (out_byte<<1) & 11111110b;	// Move all previous bits left
			out_byte++;				// Add a one to the right-most bits
			}
		else if (input=='0'){		// Character is a 0
			out_byte = (out_byte<<1) & 11111110b;	// Move all previous bits left
			}
		else {						// If the character is not a valid 1 or 0
			out_byte = 0;			// Return a zero
			break;					// Stop asking for bits
			}
		}
		return out_byte;			// Return the user's Short
}
	
	
	
////////////////////////////////////////////
//	Function: usart_putByte
//	Purpose:  puts in a byte in binary form
//  Input:    The byte to output
//  Output:   Void
////////////////////////////////////////////
void usart_putByte(char reg) {
	if (reg.7 ==1) {usart_putc('1');}
	else {usart_putc('0');}
	
	if (reg.6 ==1) {usart_putc('1');}
	else {usart_putc('0');}
	
	if (reg.5 ==1) {usart_putc('1');}
	else {usart_putc('0');}
	
	if (reg.4 ==1) {usart_putc('1');}
	else {usart_putc('0');}
	
	if (reg.3 ==1) {usart_putc('1');}
	else {usart_putc('0');}
	
	if (reg.2 ==1) {usart_putc('1');}
	else {usart_putc('0');}
	
	if (reg.1 ==1) {usart_putc('1');}
	else {usart_putc('0');}
	
	if (reg.0 ==1) {usart_putc('1');}
	else {usart_putc('0');}
}